const mongoose = require('mongoose');
const moment = require('moment-timezone');
const get = require('lodash.get');
const set = require('lodash.set');

const findPaths = (schema, options) => {
  const SchemaDate = mongoose.Schema.Types.Date;

  return Object.keys(schema.paths).filter((path) => {
    if (options.paths.length) return options.paths.indexOf(path) !== -1;
    return schema.paths[path] instanceof SchemaDate;
  });
};

/**
 *
 * @param {Object} schema
 * @param {Object} options Options object
 * @param {String[]} options.paths Schema paths
 */
exports.timeZone = function(schema, options = {}) {
  options = Object.assign({
    paths: [],
  }, options);

  // console.log('options', options)

  const paths = findPaths(schema, options);
  const offset = moment().utcOffset() * 60 * 1000;

  function fixOffset(doc, add) {
    // console.group('fixOffset');
    // console.log('add参数', add)
    paths.forEach((path) => {
      // console.group('循环 Date属性的paths', path);
      const date = get(doc, path);
      // console.log('修正显示时间之前', date)
      if (date) {
        const modifier = add ? 1: -1;
        const fixedOffset = date.valueOf() + (modifier * offset);
        // 对 createdAt 无效
        set(doc, path, new Date(fixedOffset));
        // console.log('修正显示时间之后', new Date(fixedOffset))
      }

      console.groupEnd();
    });
    // console.log('最终doc', doc)
    console.groupEnd();

  }

  function subtractOffset(docs, next) {
    const documents = [];

    if (!docs) return next();
    // console.log('docs之前', docs)
    if (!Array.isArray(docs)) {
      documents.push((docs.constructor.name === 'model') ? docs : this);
    } else {
      documents.push(...docs);
    }
    documents.forEach(result => fixOffset(result, true));
    next();

    // console.log('docs之后', docs)
  }

  // schema.pre('save', addOffset);
  schema.post('save', subtractOffset);
  schema.post('find', subtractOffset);
  schema.post('findOne', subtractOffset);
  schema.post('findOneAndUpdate', subtractOffset);
}
